#!/usr/bin/env /usr/bin/python

import os, pygame,math,random, sys
from pygame.locals import *

#############################################################
### Configurable Parameters #################################

# Probability of mine appearance in cell
MINE_PROBABILITY = 25

# Probability for each mine type
# Make sure it sums to 100!
MINE_1_PROBABILITY = 30
MINE_2_PROBABILITY = 50
MINE_3_PROBABILITY = 20

# Threat modifier to be added to mines in far neighborhood
FAR_THREAT_MOD = -1

### End of Configurable Parameters ##########################
#############################################################

VERSION = "0.1.2"

# This is the rectangular size of the hexagon tiles.
TILE_WIDTH = 38
TILE_HEIGHT = 41

# This is the distance in height between two rows.
ROW_HEIGHT = 31

# This value will be applied to all odd rows x value.
ODD_ROW_X_MOD = 19

# This is the size of the square grid that will help us convert pixel locations to hexagon map locations.
GRID_WIDTH = 38
GRID_HEIGHT = 31

# Defaults
MAX_MINE_POWER = 3
MINEFIELD_X_SIZE = 16
MINEFIELD_Y_SIZE = 15

# Lists of appropriate modifiers to add to current cell to get close and far neighborhood
EVEN_ROW_NEIGHBORHOOD=[(-1,-1),(0,-1),(-1,0),(1,0),(-1,1),(0,1)]
ODD_ROW_NEIGHBORHOOD=[(0,-1),(1,-1),(-1,0),(1,0),(0,1),(1,1)]
EVEN_ROW_FAR_NEIGHBORHOOD=[(-1,-2),(0,-2),(1,-2),(-2,0),(2,0),(-1,2),(0,2),(1,2),(-2,-1),(1,-1),(-2,1),(1,1)]
ODD_ROW_FAR_NEIGHBORHOOD=[(-1,-2),(0,-2),(1,-2),(-2,0),(2,0),(-1,2),(0,2),(1,2),(-1,-1),(2,-1),(-1,1),(2,1)]

X = 0
Y = 1

CLOCK_TICK=30

# This is the modification tables for the square grid.

a1=(-1,-1)
b1=(0,0)
c1=(0,-1)

gridEvenRows = [
[a1,a1,a1,a1,a1,a1,a1,a1,a1,a1,a1,a1,a1,a1,a1,a1,a1,a1,b1,b1,c1,c1,c1,c1,c1,c1,c1,c1,c1,c1,c1,c1,c1,c1,c1,c1,c1,c1],
[a1,a1,a1,a1,a1,a1,a1,a1,a1,a1,a1,a1,a1,a1,a1,a1,b1,b1,b1,b1,b1,b1,c1,c1,c1,c1,c1,c1,c1,c1,c1,c1,c1,c1,c1,c1,c1,c1],
[a1,a1,a1,a1,a1,a1,a1,a1,a1,a1,a1,a1,a1,a1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,c1,c1,c1,c1,c1,c1,c1,c1,c1,c1,c1,c1,c1,c1],
[a1,a1,a1,a1,a1,a1,a1,a1,a1,a1,a1,a1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,c1,c1,c1,c1,c1,c1,c1,c1,c1,c1,c1,c1],
[a1,a1,a1,a1,a1,a1,a1,a1,a1,a1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,c1,c1,c1,c1,c1,c1,c1,c1,c1,c1],
[a1,a1,a1,a1,a1,a1,a1,a1,a1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,c1,c1,c1,c1,c1,c1,c1,c1,c1],
[a1,a1,a1,a1,a1,a1,a1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,c1,c1,c1,c1,c1,c1,c1],
[a1,a1,a1,a1,a1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,c1,c1,c1,c1,c1],
[a1,a1,a1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,c1,c1,c1],
[a1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,c1],
[b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1],
[b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1],
[b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1],
[b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1],
[b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1],
[b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1],
[b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1],
[b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1],
[b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1],
[b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1],
[b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1],
[b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1],
[b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1],
[b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1],
[b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1],
[b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1],
[b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1],
[b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1],
[b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1],
[b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1],
[b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1]
]

a2=(-1,0)
b2=(0,-1)
c2=(0,0)

gridOddRows = [
[a2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,c2],
[a2,a2,a2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,c2,c2,c2],
[a2,a2,a2,a2,a2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,c2,c2,c2,c2,c2],
[a2,a2,a2,a2,a2,a2,a2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,c2,c2,c2,c2,c2,c2,c2],
[a2,a2,a2,a2,a2,a2,a2,a2,a2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,c2,c2,c2,c2,c2,c2,c2,c2,c2],
[a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2],
[a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2],
[a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2],
[a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,b2,b2,b2,b2,b2,b2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2],
[a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,b2,b2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2],
[a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2],
[a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2],
[a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2],
[a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2],
[a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2],
[a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2],
[a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2],
[a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2],
[a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2],
[a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2],
[a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2],
[a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2],
[a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2],
[a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2],
[a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2],
[a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2],
[a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2],
[a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2],
[a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2],
[a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2],
[a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,a2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2,c2]
]

class Cell:
	"""
	Class representing cell data. 
	"""
	def __init__(self):
		self.state = "closed"	# Cell state. "open" or "close"
		self.mine = 0		# Mine power in cell. 0 - no mine
		self.flag = 0		# User flag on cell. 0 - no cell
		self.threat = 0		# Sum of mine powers in close neighborhood
		self.far_threat = 0	# Threat from far neighborhood. See MineField.calc_far_threat()
	
	def get_state(self): return self.state
	def get_mine(self): return self.mine
	def get_flag(self): return self.flag
	def get_threat(self): return self.threat
	def get_far_threat(self): return self.far_threat
	
	def set_threat(self, new_threat):
		self.threat = new_threat
		return self.threat
	
	def set_far_threat(self, new_far_threat):
		self.far_threat = new_far_threat
		return self.far_threat
	
	def set_flag(self, new_flag): 
		self.flag = new_flag % (MAX_MINE_POWER + 1)
		return self.flag 
	
	def set_mine(self, new_mine): 
		self.mine = new_mine % (MAX_MINE_POWER + 1)
		return self.mine 
	
	def set_state(self, new_state="open"): 
		self.state = new_state
		return self.state 
	

class MineField:
	"""
	Represents hexagonal mine field.
	Attributes:
		field[][]	-- 2-dimensional array of cells
	"""
	
	def __init__(self):
		self.field = []
		self.free_cell_number = 0
		self.open_cell_count = 0
		
		for x in range(MINEFIELD_X_SIZE):
			col = []
			for y in range(MINEFIELD_Y_SIZE):
				new_cell = Cell()
				randval = random.randint(1,100)
				if randval <= MINE_PROBABILITY:
					randval = random.randint(1,100)
					if randval <= MINE_1_PROBABILITY:
						new_cell.set_mine(1)
					elif randval <= MINE_1_PROBABILITY + MINE_2_PROBABILITY:
						new_cell.set_mine(2)
					else:
						new_cell.set_mine(3)
					#print "M(%d,%d) -> %d" % (x+1,y+1,new_cell.get_mine())
				else:
					self.free_cell_number += 1;	
				col.append(new_cell)
			self.field.append(col)
			
		for x in range(MINEFIELD_X_SIZE):
			for y in range(MINEFIELD_Y_SIZE):
				self.field[x][y].set_threat(self.calc_threat((x,y)))
				self.field[x][y].set_far_threat(self.calc_far_threat((x,y)))
	
	def get_cell(self, position):
		if self.check_position(position): return self.field[position[X]][position[Y]]
		else: return None
	
	def get_neighborhood_list(self, position):
		if not self.check_position(position): return None
		ret = []
		
		if position[Y]&1:
			n_list = ODD_ROW_NEIGHBORHOOD
		else:
			n_list = EVEN_ROW_NEIGHBORHOOD
			
		for (x_off, y_off) in n_list:
			n_cell = (position[X]+x_off, position[Y]+y_off)
			if self.check_position(n_cell):
				ret.append(n_cell)
		
		return ret
	
	def get_neighborhood(self, position):
		if not self.check_position(position): return None
		ret = []
		
		for cur_pos in self.get_neighborhood_list(position):
			ret.append(self.get_cell(cur_pos))
		return ret
	
	def calc_threat(self, position):
		ret = 0
		for cur_cell in self.get_neighborhood(position):
			ret = ret + cur_cell.get_mine();
		#print "T(%d,%d) -> %d\n" % (position[X]+1, position[Y]+1, ret)
		return ret
	
	def get_far_neighborhood_list(self, position):
		if not self.check_position(position): return None
		ret = []
		
		if position[Y]&1:
			n_list = ODD_ROW_FAR_NEIGHBORHOOD
		else:
			n_list = EVEN_ROW_FAR_NEIGHBORHOOD
			
		for (x_off, y_off) in n_list:
			n_cell = (position[X]+x_off, position[Y]+y_off)
			if self.check_position(n_cell):
				ret.append(n_cell)
		
		return ret
	
	def get_far_neighborhood(self, position):
		if not self.check_position(position): return None
		ret = []
		
		for cur_pos in self.get_far_neighborhood_list(position):
			ret.append(self.get_cell(cur_pos))
		return ret
	
	def calc_far_threat(self, position):
		ret = 0
		for cur_cell in self.get_far_neighborhood(position):
			if cur_cell.get_mine() > 0: ret = ret + cur_cell.get_mine() + FAR_THREAT_MOD
		#print "FT(%d,%d) -> %d\n" % (position[X]+1, position[Y]+1, ret)
		return ret
	
	
	def check_position(self, position):
		if position[X] >= 0 and position[X] < MINEFIELD_X_SIZE and position[Y] >= 0 and position[Y] < MINEFIELD_Y_SIZE:
			return True
		else:
			return False
		
	def open_cell(self, position):
		if not self.check_position(position): return None
		if self.get_cell(position).get_state() == "opened" or self.get_cell(position).get_flag() != 0 : return True
		self.get_cell(position).set_state("opened")
		self.open_cell_count += 1
		if self.get_cell(position).get_mine ()!= 0:
			return "RIP"
		if self.get_cell(position).get_threat() == 0:
			for pos in self.get_neighborhood_list(position):
				self.open_cell(pos)
		if self.open_cell_count == self.free_cell_number:
			return "VICTORY"
		return True
	
	def flag_cell(self, position):
		if not self.check_position(position): return None
		self.get_cell(position).set_flag(self.get_cell(position).get_flag() + 1)
		return True

class HexaMine:

    def pixelToHexMap(self,x,y):
        """
        Converts a pixel location to a location on the hexagon map.
        """        

        # Get the square location in our help grid.
        gridX = x/GRID_WIDTH
        gridY = y/GRID_HEIGHT
            
        # Calculate the pixel location within that square
        gridPixelX = x%GRID_WIDTH
        gridPixelY = y%GRID_HEIGHT

        # Update the gridRect to show the correct location in the grid
        self.gridRect.topleft = (gridX*GRID_WIDTH,gridY*GRID_HEIGHT)
       
        # Apply the modifiers to get the correct hexagon map location.
        if gridY&1:
            # Odd rows
            hexMapX=gridX+gridOddRows[gridPixelY][gridPixelX][0]
            hexMapY=gridY+gridOddRows[gridPixelY][gridPixelX][1]
        else:
            # Even rows
            hexMapX=gridX+gridEvenRows[gridPixelY][gridPixelX][0]
            hexMapY=gridY+gridEvenRows[gridPixelY][gridPixelX][1]

        return (hexMapX,hexMapY)

    def hexMapToPixel(self,mapX,mapY):
        """
        Returns the top left pixel location of a hexagon map location.
        """
        if mapY & 1:
            # Odd rows will be moved to the right.
            return (mapX*TILE_WIDTH+ODD_ROW_X_MOD,mapY*ROW_HEIGHT)
        else:
            return (mapX*TILE_WIDTH,mapY*ROW_HEIGHT)
            

    def drawMap(self, special=None):       
        """
        Draw the tiles.
        """
        #fnt = pygame.font.Font(pygame.font.get_default_font(),12)
	fnt = pygame.font.Font("./FreeSansBold.ttf",14)

        self.mapimg = pygame.Surface((640,480),1)
        self.mapimg= self.mapimg.convert()
        self.mapimg.fill((104,104,104))

        for x in range(16):
            for y in range(15):
                # Get the top left location of the tile.
                pixelX,pixelY = self.hexMapToPixel(x,y)

                # Blit the tile to the map image.
		if not special:
			if self.minefield.get_cell((x,y)).get_state() == "closed":
				if self.minefield.get_cell((x,y)).get_flag() == 0:
					self.mapimg.blit(self.tile,(pixelX,pixelY))
				elif self.minefield.get_cell((x,y)).get_flag() == 1:
					self.mapimg.blit(self.flag1,(pixelX,pixelY))
				elif self.minefield.get_cell((x,y)).get_flag() == 2:
					self.mapimg.blit(self.flag2,(pixelX,pixelY))
				elif self.minefield.get_cell((x,y)).get_flag() == 3:
					self.mapimg.blit(self.flag3,(pixelX,pixelY))
			elif self.minefield.get_cell((x,y)).get_state() == "opened":
				self.mapimg.blit(self.tile_open,(pixelX,pixelY))
		else:
			#mine_tile = special == "RIP" and self.tile_rip or self.tile_vic
			if self.minefield.get_cell((x,y)).get_mine() == 0:
				self.mapimg.blit(self.tile_open,(pixelX,pixelY))
			else:
				self.mapimg.blit(special == "RIP" and self.tile_rip or self.tile_vic,(pixelX,pixelY))
			
		mark = None
                # Show the hexagon map location in the center of the tile.
                #mark = fnt.render("%d,%d" % (x,y), 0, (0xff,0xff,0xff))
		if not special:
			if self.minefield.get_cell((x,y)).get_state() == "opened":
				mark = fnt.render("%d/%d" % (self.minefield.get_cell((x,y)).get_threat(),self.minefield.get_cell((x,y)).get_far_threat()), 0, (0xff,0xff,0xff))
			elif self.minefield.get_cell((x,y)).get_flag() != 0:
				mark = fnt.render("%d" % (self.minefield.get_cell((x,y)).get_flag()), 0, (0xff,0xff,0xff))
		else:
			if self.minefield.get_cell((x,y)).get_mine() == 0:
				mark = fnt.render("%d/%d" % (self.minefield.get_cell((x,y)).get_threat(),self.minefield.get_cell((x,y)).get_far_threat()), 0, (0xff,0xff,0xff))	
			else:
				mark = fnt.render("%d" % (self.minefield.get_cell((x,y)).get_mine()), 0, (0xff,0xff,0xff))
                if mark:
			lrect=mark.get_rect()
                	lrect.center = (pixelX+(TILE_WIDTH/2),pixelY+(TILE_HEIGHT/2))   
                	self.mapimg.blit(mark,lrect.topleft)
                    
    def loadTiles(self):
        """
        Load the tile and the cursor.
        """
        self.tile = pygame.image.load("./hextile.png").convert()
        self.tile.set_colorkey((0x80, 0x00, 0x80), RLEACCEL)                

        self.cursor = pygame.image.load("./hexcursor.png").convert()
        self.cursor.set_colorkey((0x80, 0x00, 0x80), RLEACCEL)                
        self.cursor_fn = pygame.image.load("./hexcursor_fn.png").convert()
        self.cursor_fn.set_colorkey((0x80, 0x00, 0x80), RLEACCEL)     
	self.cursorPos = self.cursor.get_rect()
	self.fn_cursorPos=[]
	
	self.flag1 = pygame.image.load("./hextile_flag_1.png").convert()
	self.flag1.set_colorkey((0x80, 0x00, 0x80), RLEACCEL)
	
	self.flag2 = pygame.image.load("./hextile_flag_2.png").convert()
	self.flag2.set_colorkey((0x80, 0x00, 0x80), RLEACCEL)
	
	self.flag3 = pygame.image.load("./hextile_flag_3.png").convert()
	self.flag3.set_colorkey((0x80, 0x00, 0x80), RLEACCEL)
	
	self.tile_open = pygame.image.load("./hextile_open.png").convert()
        self.tile_open.set_colorkey((0x80, 0x00, 0x80), RLEACCEL)
	
	self.tile_rip = pygame.image.load("./hextile_mine.png").convert()
	self.tile_rip.set_colorkey((0x80, 0x00, 0x80), RLEACCEL)
	
	self.tile_vic = pygame.image.load("./hextile_vic.png").convert()
	self.tile_vic.set_colorkey((0x80, 0x00, 0x80), RLEACCEL)
	
    def init(self):
        """
        Setup the screen etc.
        """
        self.screen = pygame.display.set_mode((640, 480),1)
        pygame.display.set_caption("HexaMine %s | ESC - Quit" % (VERSION))

	self.minefield = MineField()

        self.loadTiles()
        self.drawMap()        

        self.gridRect = pygame.Rect(0,0,GRID_WIDTH,GRID_HEIGHT)
	
	self.cur_cursor_cell = None
	self.redraw = True

    def setCursor(self,x,y):
        """
        Set the hexagon map cursor.
        """
        mapX,mapY = self.pixelToHexMap(x,y)
        
	if self.cur_cursor_cell != (mapX,mapY):
		self.cur_cursor_cell = (mapX,mapY)
		self.redraw = True
		pixelX,pixelY = self.hexMapToPixel(mapX,mapY)
		self.cursorPos.topleft = (pixelX,pixelY)
		
		self.fn_cursorPos=[]
		
		if self.minefield.check_position((mapX,mapY)) and self.minefield.get_cell((mapX,mapY)).get_state() == "opened":
			for fnc in self.minefield.get_far_neighborhood_list((mapX,mapY)):
				trect = pygame.Rect(0,0,GRID_WIDTH,GRID_HEIGHT)
				pixelX,pixelY = self.hexMapToPixel(fnc[X],fnc[Y])
				trect.topleft = (pixelX,pixelY)
				self.fn_cursorPos.append(trect)

    def mainLoop(self):    
        pygame.init()    

        self.init()

        clock = pygame.time.Clock()
                        
        showGridRect = False

        while 1:
		clock.tick(CLOCK_TICK)
		# DRAWING
		if self.redraw: 
			self.drawMap()
			self.screen.blit(self.mapimg, (0, 0))
			self.screen.blit(self.cursor,self.cursorPos)
			for cp in self.fn_cursorPos:
				self.screen.blit(self.cursor_fn,cp)               
			if showGridRect:
				pygame.draw.rect(self.screen, (0xff,0xff,0xff), self.gridRect, 1)
			
			pygame.display.flip()
		self.redraw = False
		for event in pygame.event.get():
                	if event.type == QUIT:
                    		return
		    	#sys.exit()
			elif event.type == KEYUP:
				if event.key == K_ESCAPE:
					return
					#sys.exit()
			elif event.type == MOUSEMOTION:
				self.setCursor(event.pos[0],event.pos[1])
		
			elif event.type == MOUSEBUTTONUP:
				loc = self.pixelToHexMap(event.pos[0],event.pos[1])
				self.redraw = True
				if self.minefield.check_position(loc):
					if event.button == 1:
						if self.minefield.get_cell(loc).get_flag() == 0:
							status = self.minefield.open_cell(loc)
							if status == "RIP" or status == "VICTORY" :
								pygame.display.set_caption("HexaMine %s | %s | ESC - Quit, SPACE/Mouse click - Continue" % (VERSION, status))
								self.drawMap(status)
								self.screen.blit(self.mapimg, (0, 0))
								pygame.display.flip()
								while 1:
									clock.tick(CLOCK_TICK)
									for event in pygame.event.get():
										if event.type == QUIT:
											return
											#sys.exit()
										elif event.type == KEYUP:
											if event.key == K_ESCAPE:
												return
											elif event.key == K_SPACE:
												return "Continue"
										elif event.type == MOUSEBUTTONUP:
											return "Continue"
								return "Continue"
								
					elif event.button == 2:
						pass
					elif event.button == 3:
						self.minefield.flag_cell(loc)
		
def main():
    g = HexaMine()
    while g.mainLoop() == "Continue": pass

 
#this calls the 'main' function when this script is executed
if __name__ == '__main__': main()
